package object_detection;

import database.MongoDBInteraction;
import object_detection.types.ObjectSet;
import object_detection.types.Point;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class ObjectDetector {


    /**
     * The starting function that creates an object set, compiles informatino, and returns it
     * TODO: need to add functionality to update Database
     * @throws FileNotFoundException
     */
    public static void startProcess() throws FileNotFoundException {

        // for now, we can just set paths to the directories that hold keyframes and featurepoint CSVs
        String feat_dir_pth = "src/main/java/vslam/KeyFramePoints";
        String keyframe_png_path = "src/main/java/vslam/KeyFrames";
        String bbox_dir_pth = "src/main/java/vslam/BoundedInfo";

        // get files
        File[] feat_CSVs = getDirFiles(feat_dir_pth);
        File[] bbox_CSVs = getDirFiles(bbox_dir_pth);

        // sort to guarantee correct order for keyframes
        Arrays.sort(feat_CSVs);
        Arrays.sort(bbox_CSVs);

        // check to make sure both file arrays have same number of keyframes (for consistency)
        if(feat_CSVs.length != bbox_CSVs.length){
            System.err.println("ERROR: features and bounding box directories have differing number of Keyframes");
        }

        /* #################################################
        In the section below, we create a new ObjectSet, and iterate over each Keyframe
         ################################################## */

        System.out.println("====> Starting Object Mapping");

        // initialize ObjectSet
        ObjectSet objSet = new ObjectSet();

        // for each keyframe, process bounding box csv and feature csv
        int NUMKF = feat_CSVs.length;
        for(int i = 0; i < NUMKF; i++){
            // process the frame
            List<ArrayList<Point>> currFramePoints = objSet.processFrame(feat_CSVs[i], bbox_CSVs[i]);

            // update the current object set
            objSet.updateObjectSet(currFramePoints);
        }

        // printing final object set for sanity
        System.out.println(objSet);
        System.out.println("====> Finished Object Mapping");

        MongoDBInteraction mdbi = new MongoDBInteraction();
        mdbi.updateObjectSet(5, objSet);
    }

    /**
     * A helper function used to pull all files out of a directory path
     * @param dir_pth
     * @return
     */
    public static File[] getDirFiles(String dir_pth){
        File[] f_arr;
        // get the csv files of each frame
        try {
            f_arr = new File(dir_pth).listFiles();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }

        // some error checking
        if(f_arr == null || f_arr.length == 0){
            System.err.println("ERROR: no feature csvs found at given path");
        }

        return f_arr;
    }

    public static void main(String[] args) throws FileNotFoundException {
        startProcess();
    }

}
